@@ -8,29 +8,34 @@ from copy import deepcopy
 import json
 import itertools
 import operator
+from operator import itemgetter
 
 import numpy as np
+
 import matplotlib.pyplot as plt
 
 import ImageContour
 
 #from list_utils:
-from np_utils import totuple,interpGen,flatten,ziptranspose
+from np_utils import ( totuple, interpGen, flatten, ziptranspose, roll,
+                       deletecases,partition, polyCirculationDirection,
+                       groupByFunction, getElementConnections,
+                       getChainsFromConnections, removeDuplicates,
+                       removeAdjacentDuplicates )
+#from func_utils
+from np_utils import compose
 #from np_utils:
-from np_utils import limitInteriorPoints,limitInteriorPointsInterpolating
+from np_utils import ( limitInteriorPoints, limitInteriorPointsInterpolating,
+                       addBorder, getValuesAroundPointInArray )
 
-def GetValuesAroundSCPoint(watershed2d,point):
-    '''Given any junction in a pixel array, get the set of unique values that surround it;
-       there are always 4 pixels around a junction
-       there is one more junction in each direction than there are pixels
-       (but one less internal junction)
-       "point" must be the index of an internal junction'''
-    x,y = point
-    if  0 < x <= watershed2d.shape[0] and  0 < y <= watershed2d.shape[1]:
-        return tuple(np.unique( watershed2d[x-1:x+1,y-1:y+1] ).tolist())
-    else:
-        print "THIS POINT IS NOT INTERIOR TO THE ARRAY; THERE ARE NOT 4 POINTS AROUND IT!"
-        return (None,None,None)
+def GetValuesAroundSCPoint(watershed2d,point,wrapX=False,wrapY=False):
+    vals = getValuesAroundPointInArray(watershed2d,point,wrapX,wrapY)
+    return ( tuple(vals.tolist()) if vals!=None else (None,None,None) )
+
+def _kwdPop(kwds,key,defaultValue):
+    '''If a dictionary has a key, pop the value and return it,
+       otherwise return defaultValue'''
+    return ( kwds.pop(key) if key in kwds else defaultValue )
 
 class SubContour(object):
     '''A class to hold the data for a single SubContour (basically a connected list of points)
@@ -167,7 +172,9 @@ class CellNetwork(object):
     def UpdateAllValues(self):
         '''Go through the values in all the subContours and collect a list of all of them
            State: Changes state of allValues, but only to to be more consistent'''
-        self.allValues = sorted(set( [ v for sc in self.subContours for v in sc.values if v!=1 ] ))
+        self.allValues = sorted(set( [ v for sc in self.subContours 
+                                         for v in sc.values
+                                         if v!=1 ] ))
     
     def GetAllPoints(self):
         '''Get a sorted set of all points in the subContours
@@ -206,7 +213,7 @@ class CellNetwork(object):
                     # Aka, there are not enough points to make a triangle...
                     problemVals.add(v)
                     problemSCs.update(scInds)
-        problemValuePairs = [self.subContours[i].values for i in problemSCs]
+        problemValuePairs = [ self.subContours[i].values for i in problemSCs ]
         return sorted(problemVals),sorted(problemValuePairs)
     
     def LimitPointsBetweenNodes(self,numInteriorPointsDict,interpolate=True,checkForDegeneracy=True):
@@ -438,15 +445,6 @@ class CellNetwork(object):
         for sc in self.subContours:
             _=sc.plot(*args,**kwds)
     
-    def cellPlot(self,*args,**kwds):
-        '''Plot the full contours in a way that can be overlaid on an imshow
-           State: Access only'''
-        contourPoints = { v:self.GetContourPoints(v) for v in self.contourOrderingByValue.keys() }
-        for v in contourPoints.keys():
-            x = [ p[0]-0.5 for p in contourPoints[v] ]
-            y = [ p[1]-0.5 for p in contourPoints[v] ]
-            _=plt.plot( y,x, *args, **kwds )
-    
     def scPlotT(self,*args,**kwds):
         '''Plot the subContours in a way that can be overlaid on a transposed imshow
            (matches closely with diagramPlot in VFMLite)
@@ -454,16 +452,25 @@ class CellNetwork(object):
         for sc in self.subContours:
             _=sc.plotT(*args,**kwds)
     
-    def cellPlotT(self,*args,**kwds):
-        '''Plot the full contours in a way that can be overlaid on a transposed imshow
-           (matches closely with diagramPlot in VFMLite)
+    def cellPlot(self,*args,**kwds):
+        '''Plot the full contours in a way that can be overlaid on an imshow
            State: Access only'''
+        plotFunction = _kwdPop( kwds, 'plotFunction', plt.plot )
+        reverseXY    = _kwdPop( kwds, 'reverseXY'   , False    )
+        
         contourPoints = { v:self.GetContourPoints(v) for v in self.contourOrderingByValue.keys() }
         for v in contourPoints.keys():
             x = [ p[0]-0.5 for p in contourPoints[v] ]
             y = [ p[1]-0.5 for p in contourPoints[v] ]
-            _=plt.plot( x,y, *args, **kwds )
-
+            x,y = (y,x) if reverseXY else (x,y)
+            _=plotFunction( y,x, *args, **kwds )
+    
+    def cellPlotT(self,*args,**kwds):
+        '''Plot the full contours in a way that can be overlaid on a transposed imshow
+           (matches closely with diagramPlot in VFMLite)
+           State: Access only'''
+        kwds['reverseXY'] = True
+        return self.cellPlot(*args,**kwds)
 
 def GetFlatDataFromCellNetwork(cn):
     '''Convert a CellNetwork with all its nested objects into flat datastructure
@@ -508,7 +515,12 @@ def SubContourListfromCVLSList(cVLS_List,startPointValues_List=[],endPointValues
             for i,cvls in enumerate(cVLS_List)]
 
 def GetCellNetwork( watershed2d,allValues=None,bgVals=(0,1),scale=1,offset=(0,0), ):
-    '''Basically a constructor for CellNetwork based on a watershed array'''
+    '''Basically a constructor for CellNetwork based on a watershed array
+       
+       watershed2d is a watershed-like 2d array (indexed block values)
+       allValues is optional and specifies which value in the array to include in the network
+       bgVals are the values ignored as background (at least 0 should always be used)
+       scale and offset define a non-rotational transform over the output points'''
     if allValues==None:
         allValues = np.unique(watershed2d)
     allValues = np.array(allValues).tolist() # force numpy arrays to lists
@@ -1154,3 +1166,208 @@ def ContourPlotFromCVLS(cVLSByFrame,frame=0):
     for cvls in cVLSByFrame[frame]:
         cvls=np.array(cvls[2])
         _=plt.plot( cvls[:,0], cvls[:,1] )
+
+
+#####################################################################
+## New section for array-based contour generation                  ##
+## Lacks some of the finesse of the ImageContour's crawling method ##
+#####################################################################
+
+def getDiffArr(arr,axis,wrap=False):
+    '''Create an array that delineates pixel-pixel boundaries along an axis'''
+    diffArr = addBorder(np.diff(arr,axis=axis),1,axis=axis)!=0
+    if wrap:
+        pass
+        if axis==0: diffArr[-1] = 0
+        else:       diffArr[:,-1] = 0
+    return diffArr
+
+def getWhWhp(arr,axis,wrap=False):
+    '''Get the start and end points of the boundary lines
+       between all differing pixels along an axis
+       (as np.where results)'''
+    lineArr = getDiffArr(arr,axis,wrap)
+    wh = np.where(lineArr)
+    p1 = ( (0,1) if axis==0 else (1,0) )
+    whp = (wh[0]+p1[0],wh[1]+p1[1])
+    return wh,whp
+
+def GetDividingLinesFromArray(arr,axis,wrap=False,whwhp=None):
+    '''Get all the lines perpendicular to axis that divide differing pixels
+       (as an array of [[x1,y1],[x2,y2]] values)'''
+    wh,whp = ( whwhp if whwhp else getWhWhp(arr,axis,wrap) )
+    edges = np.transpose([wh,whp],(2,0,1))
+    return edges
+
+def GetValuePairsFromArray(arr,axis,wrap=False,whwhp=None):
+    '''Given an array and an axis, find all pairs of adjacent points
+       Optionally takes a whwhp parameter (generated by getWhWhp) to ensure that
+       values generated by this function correlate to edges generated by GetDividingLinesFromArray'''
+    wh,whp = ( whwhp if whwhp else getWhWhp(arr,axis,wrap) )
+    def s(x,y): return ((x,y) if axis==0 else (y,x)) # Flip arguments when axis==1
+    sA,s1A,s1m1 = slice(None),slice(1,None),slice(1,-1) # shorthand for slices [:], [1:], and [1:-1]
+    
+    arrB = addBorder(arr) # add 1 pixel border to each side along axis
+    if wrap:  # wrap the sides around
+        arrB[s(0,s1m1)]  = arr[s(-1,sA)]
+        arrB[s(-1,s1m1)] = arr[s(0,sA)]
+    startValue  = arrB[s(sA,s1A)][wh]
+    endValue = arrB[s(s1A,sA)][whp]
+    return np.transpose([startValue,endValue]) # valuePairs
+
+def GetDividingLinesAndValuePairsFromArray(arr,axis,wrap=False):
+    '''Return the set of lines (in [[x1,y1],[x2,y2]] format) that divide pixels
+       of different value as well as a list of all the actual pixel values'''
+    whwhp = getWhWhp(arr,axis,wrap)
+    edgesAxis      = GetDividingLinesFromArray(arr,axis,wrap,whwhp)
+    valuePairsAxis = GetValuePairsFromArray(arr,axis,wrap,whwhp)
+    return edgesAxis,valuePairsAxis
+
+def GetEdgesAndValuePairsFromArray(arr,wrapX=False,wrapY=False):
+    '''Just like GetDividingLinesAndValuePairsFromArray but applies
+       it over both x and y axes'''
+    evpXY = [ GetDividingLinesAndValuePairsFromArray(arr,axis,wrap)
+             for axis,wrap in [(0,wrapX),(1,wrapY)] ]
+    edges,valuePairs = map( np.concatenate,ziptranspose(evpXY) )
+    return edges,valuePairs
+
+def GetEdgeGroups(edges,valuePairs,eliminateSameCellBoundaries=True):
+    '''Get a list of lines between each cell pair
+       
+       edges and valuePairs are generated by GetEdgesAndValuePairsFromArray
+       
+       The "eliminateSameCellBoundaries" option eliminates lines where the values
+       on either side are acutally the same; this should only occur at wrapped edges.'''
+    edgeGroups = groupByFunction( zip(edges.tolist(),np.sort(valuePairs).tolist()),
+                                  compose(tuple,itemgetter(1)), itemgetter(0) )
+    return ( { k:v for k,v in edgeGroups.iteritems() if k[0]!=k[1] }
+             if eliminateSameCellBoundaries else
+             edgeGroups )
+
+def GetAmbiguousCrossingPoints(arrayWithBorder):
+    '''Find all spots in an array where an ambiguous intersection occurs,
+       namely a point surrrounded by a checkerboard pattern, i.e.:
+       ****
+       *AB*
+       *BA*
+       ****
+       '''
+    a = arrayWithBorder
+    return np.transpose(np.where(np.all([ a[:-1,:-1] == a[1:,1:],
+                                          a[:-1,1:]  == a[1:,:-1],
+                                          a[:-1,:-1] != a[1:,:-1],
+                                        ], axis=0 )))
+
+def GetContourOrderingByValue(cellIDs,subContourValues,subContourPoints):
+    pointIndicesByCellID = { cellID:[ i for i,values in enumerate(subContourValues)
+                                        if cellID in values ]
+                            for cellID in cellIDs }
+    # Not used:
+    ##edgeGroupsByCellID = { cellID: [ subContourPoints[i] for i in pointIndicesByCellID[cellID] ]
+    ##                      for cellID in cellIDs }
+    contourOrderingByValue = {}
+    for cellID in cellIDs:
+        scInds = pointIndicesByCellID[cellID]
+        scToEndPointConnections = [ ( i, subContourPoints[i][j] )
+                                   for i in scInds
+                                   for j in (0,-1) ]
+        conns = getElementConnections(scToEndPointConnections)
+        chainsOut = getChainsFromConnections(conns)[0]
+        chainsOut = ( chainsOut[:-1] if chainsOut[0]==chainsOut[-1] else
+                      chainsOut )
+        chainsOut = ( roll(chainsOut) if hasattr(chainsOut[0],'__iter__') else
+                      chainsOut )
+        scOrder, scEndPts = ziptranspose(partition(chainsOut,2))
+        scDirection = [ ( subContourPoints[sco][-1]==scEndPts[i] ) # test for head to tail connections
+                       for i,sco in enumerate(scOrder) ]
+        chainDirection = polyCirculationDirection(scEndPts)
+        if chainDirection==-1:
+            scOrder = scOrder[::-1]
+            scDirection = [ (not i) for i in scDirection ][::-1]
+        contourOrderingByValue[cellID] = zip(scOrder,scDirection)
+    return contourOrderingByValue
+
+
+def GetCellNetwork_NEW( watershed2d,allValues=None,bgVals=(0,1),scale=1,offset=(0,0),
+                        wrapX=False, collapsePoles=False, deleteZeroContours=True ):
+    '''Basically a constructor for CellNetwork based on a watershed array
+       
+       This function gives an equivalent output to GetCellNetwork, but uses an
+           array-based methodology to get the main data and then uses a
+           chaining scheme to construct subContours and contour ordering
+           (the original method uses a crawling scheme instead, which
+            obviated the need for chaining)
+       This method is ~70% slower but has more available options
+       
+       watershed2d is a watershed-like 2d array (indexed block values)
+       allValues is optional and specifies which value in the array to include in the network
+       bgVals are the values ignored as background (at least 0 should always be used)
+       scale and offset define a non-rotational transform over the output points
+       
+       Generally, these default options should be used for the other 3 options:
+          wrapX=False, collapsePoles=False, deleteZeroContours=True
+       
+       Other useful modes are:
+         * include boundary mode:
+             wrapX=False, collapsePoles=False, deleteZeroContours=False
+         * cylindrical mode (include outer boundary and wrap around in the x-dimension):
+             wrapX=True, collapsePoles=False, deleteZeroContours=False
+         * shperical mode (wrap x and collapse y):
+             wrapX=True, collapsePoles=True, deleteZeroContours=True
+       '''
+    arr = watershed2d # synonym
+    cellIDs = deletecases( ( np.unique(arr)
+                             if allValues==None else         # rebuild allValues if None is passed
+                             np.array(allValues) ).tolist(), # force numpy array to list
+                           bgVals )                          # remove background values
+    
+    # Get Edges
+    edges,valuePairs = GetEdgesAndValuePairsFromArray(arr,wrapX=wrapX,wrapY=False)
+    
+    # (convert points in e to wrapped coordinates)
+    if wrapX:
+        wh=np.where(edges[:,:,0]==arr.shape[0])
+        edges[wh[0],wh[1],0] = 0
+        edges[wh[0]] = edges[wh[0]][:,::-1] # flip so the 0 comes first (aka sorted properly...)
+    
+    #if wrapY:
+    #    # Is this right yet?
+    #    wh=np.where(e[:,:,1]==arr.shape[1])
+    #    edges[wh[0],wh[1],1] = 0
+    #    edges[wh[0]] = edges[wh[0]][:,::-1] # flip so the 0 comes first (aka sorted properly...)
+
+    # Ambiguous point testing step
+    ambiguousCrossingPoints = GetAmbiguousCrossingPoints(arr)
+    assert len(ambiguousCrossingPoints)==0, 'Ambiguous crossing points at: '+str(ambiguousCrossingPoints.tolist())
+
+    # SC step: Group edges by value pair and then form chains that divide each cell pair
+    edgeGroups = GetEdgeGroups(edges,valuePairs,eliminateSameCellBoundaries=True)
+    cellBoundariesMulti = { k:getChainsFromConnections( getElementConnections(totuple(v)) )
+                            for k,v in edgeGroups.items() }
+    assert all( len(i)==1 for i in cellBoundariesMulti.values() ),'Some cells touch in multiple places'
+    cellBoundaries = { k:v[0] for k,v in cellBoundariesMulti.iteritems() } # We should be able to deal with one chain per cell pair
+    scPtsByPairID = cellBoundaries
+    if collapsePoles:
+        # Eliminate all edges on the poles:
+        # 1. Replace points (0,x)->(0,0) and (last,x)->(last,0)
+        # 2. Merge any contiguous polar points together using removeAdjacentDuplicates
+        _collapsePoles = lambda pts: [ ( (0,0)            if p[1]==0 else
+                                         (0,arr.shape[1]) if p[1]==arr.shape[1] else
+                                         p )
+                                      for p in pts ]
+        scPtsByPairID = { k : removeAdjacentDuplicates(_collapsePoles(pts))
+                         for k,pts in scPtsByPairID.iteritems() }
+    if deleteZeroContours:
+        scPtsByPairID = { k:pts for k,pts in scPtsByPairID.iteritems()
+                                if 0 not in k }
+
+    subContourValues,subContourPoints = ziptranspose(scPtsByPairID.items())
+    contourOrderingByValue = GetContourOrderingByValue(cellIDs,subContourValues,subContourPoints)
+    subContours = [ SubContour( points=points,
+                                values=values,
+                                startPointValues = GetValuesAroundSCPoint( arr, points[0], wrapX=True ),
+                                endPointValues   = GetValuesAroundSCPoint( arr, points[-1], wrapX=True ) )
+                   for values,points in zip(subContourValues,subContourPoints) ]
+    return CellNetwork( subContours=subContours,
+                        contourOrderingByValue=contourOrderingByValue,
+                        allValues=cellIDs )
